Package: goseq
Title: Gene Ontology analyser for RNA-seq and other length biased data
Version: 1.65.0
Date: 2025-07-22
Authors@R: 
    c(
        person(
            given = "Matthew", family = "Young", role = c("aut"), 
            email = "my4@sanger.ac.uk"
        ),
        person(
            given = "Nadia", family = "Davidson", role = c("aut"), 
            email = "nadia.davidson@mcri.edu.au"
        ),
        person(
            given = "Federico", family = "Marini", role = c("ctb", "cre"), 
            email = "marinif@uni-mainz.de", comment = c(ORCID = "0000-0003-3252-7758")
        )
    )
Description: Detects Gene Ontology and/or other user defined categories
    which are over/under represented in RNA-seq data.
Depends: R (>= 2.11.0), BiasedUrn, geneLenDataBase (>= 1.9.2)
Imports: mgcv, graphics, stats, utils, AnnotationDbi, GO.db,
        BiocGenerics, methods, rtracklayer, GenomicFeatures, Seqinfo
Suggests: edgeR, org.Hs.eg.db
License: LGPL (>= 2)
URL: https://github.com/federicomarini/goseq
BugReports: https://github.com/federicomarini/goseq/issues
biocViews: ImmunoOncology, Sequencing, GO, GeneExpression,
        Transcription, RNASeq, DifferentialExpression, Annotation,
        GeneSetEnrichment, KEGG, Pathways, Software
RoxygenNote: 7.3.1
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/goseq
git_branch: devel
git_last_commit: 8a3353b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:00:25 UTC; biocbuild
Author: Matthew Young [aut],
  Nadia Davidson [aut],
  Federico Marini [ctb, cre] (ORCID:
    <https://orcid.org/0000-0003-3252-7758>)
Maintainer: Federico Marini <marinif@uni-mainz.de>
