# Generated by roxygen2: do not edit by hand

export(getgo)
export(getlength)
export(goseq)
export(makespline)
export(nullp)
export(plotPWF)
export(supportedOrganisms)
import(geneLenDataBase)
importFrom(AnnotationDbi,as.list)
importFrom(AnnotationDbi,select)
importFrom(BiasedUrn,dWNCHypergeo)
importFrom(BiasedUrn,pWNCHypergeo)
importFrom(BiocGenerics,relist)
importFrom(BiocGenerics,toTable)
importFrom(GO.db,GO.db)
importFrom(GenomicFeatures,transcriptLengths)
importFrom(Seqinfo,`genome<-`)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(methods,show)
importFrom(mgcv,Predict.matrix)
importFrom(mgcv,gam)
importFrom(mgcv,mono.con)
importFrom(mgcv,pcls)
importFrom(mgcv,s)
importFrom(mgcv,smoothCon)
importFrom(rtracklayer,browserSession)
importFrom(rtracklayer,getTable)
importFrom(rtracklayer,ucscGenomes)
importFrom(rtracklayer,ucscTableQuery)
importFrom(stats,approx)
importFrom(stats,binomial)
importFrom(stats,dhyper)
importFrom(stats,median)
importFrom(stats,phyper)
importFrom(stats,runif)
importFrom(utils,data)
importFrom(utils,installed.packages)
importFrom(utils,tail)
